#! /usr/bin/env python
from __future__ import division
import os
import re
import math
import copy

pjoin = os.path.join

def get_info(path, info_base=None, key='100tev'):
    if info_base is None:
        info = {"cross": 0.0,
            "error":0.0,
            "power": 1,
            "scUp":0,
            "scDwn":0,
            "pdfUp":0,
            "pdfDwn":0,
            "cross_8tev": None,
            "cross_100tev": None,}
    else:
        info=copy.copy(info_base)

    
    print "Treating info in '%s'..."%path
    for line in open(pjoin(path, "crossx.html")):
        if "results.html" not in line:
            continue
        oneline = line.split("<a")[1]
        oneline = oneline.split(">")
        cross= float(oneline[1].split("<")[0])
        error = float(oneline[3].split("<")[0])
        
        cross, power = ("%e" % (cross,)).split("e")
        cross = float(cross)
        power = int(power)
        # Put results in femtobarms
        power += 3
        if cross < 0:
            cross = 10 * cross
            power -=1
    
        if cross*10**power < 100.0 and cross*10**power > 0.1:
            cross = cross*10**power
            power = 0
 
        error, power2 = ("%e" % (error,)).split("e")
        error = float(error)
        power2 = int(power2)
        # Put results in femtobarms
        power2 += 3
        error = error * 10**(power2-power)
        
        info['cross'] = cross
        info['error'] = error
        info['power'] = power
        break

    min_scale = 0
    max_scale = 0
    min_pdf = None
    max_pdf = None
    # Results in femtobarns
    rescale = 1.0e3
    pdfs =[]
    #print path
    for line in open(pjoin(path, "tag_1_parton_syscalc.log")):
        if line.startswith("scale fact"):
            new_scale = rescale*float(line.split()[-1])
            if min_scale ==0 or new_scale < min_scale:
                min_scale = new_scale
            if max_scale ==0 or new_scale > max_scale:
                max_scale = new_scale
        if line.startswith("pdf reweighted"):
            pdfs.append(rescale*float(line.split()[-1]))

        if line.startswith("min/max"):
            data = line.split(":")[1].split()
            min_pdf, max_pdf = rescale*float(data[0]), rescale*float(data[1])

    # Use the hessian method.
    if pdfs:
        cntrl_val  = pdfs[0]
        pdf_up = 0
        pdf_down = 0
        for i in range(int((len(pdfs)-1)/2)):
            pdf_up   += max(0.0,pdfs[2*i+1]-cntrl_val,
                             pdfs[2*i+2]-cntrl_val)**2
            pdf_down += max(0.0,cntrl_val-pdfs[2*i+1],
                             cntrl_val-pdfs[2*i+2])**2
        #print path
        #print min_pdf, max_pdf
        max_pdf = (cntrl_val + math.sqrt(pdf_up)/2)
        min_pdf = (cntrl_val - math.sqrt(pdf_down)/2)
        #print min_pdf, max_pdf, cntrl_val


    cross = info["cross"]*10**info["power"]
    info['cross_%s'%key] = cross    
    #print cross
    if min_pdf is None or max_pdf is None:
        min_pdf = cross
        max_pdf = cross
    info["scUp"] = round(1000.0*(max_scale - cross)/cross)/10.0
    info["scDwn"] = round(1000.0*(min_scale - cross)/cross)/10.0
    info["pdfUp"] = round(1000.0*(max_pdf - cross)/cross)/10.0
    info["pdfDwn"] = round(1000.0*(min_pdf - cross)/cross)/10.0
    for elem in ["scUp","scDwn","pdfUp","pdfDwn"]:
        info[elem] = '+%.0f'%abs(info[elem]) if info[elem]>= 0 else '-%.0f'%abs(info[elem])

    info["cross"] = '%4.3g'%info["cross"]
    info["cross"] = info["cross"]+'0'*info["cross"].count(' ')
    info["cross"] = info["cross"].replace(' ','')
    info["error"] = '%5.1g'%info["error"]
    info["error"] = info["error"]+'0'*info["error"].count(' ')
    info["error"] = info["error"].replace(' ','')

    print len(info["cross"])+len(info["error"])
    if len(info["cross"])+len(info["error"])+1 == 10:
        ghost = '\, \cdot 10^{+0}d'
    elif len(info["cross"])+len(info["error"])+1 == 11:
        ghost = ' 10^{+0}d'
    elif len(info["cross"])+len(info["error"])+1 == 12:
        ghost = '0^{+0}d'

    
    # Turn off MC error
    info["cross"] = '%s'%info["cross"]+('\cdot 10^{%s%i}'%\
      ('+' if info['power']>0 else '-', abs(info['power'])) if info['power']!=0 \
                              else '\phantom{%s}'%ghost)
#    info["cross"] = '%s \pm %s'%(info["cross"],info["error"])+('\, \cdot 10^{%s%i}'%\
#      ('+' if info['power']>0 else '-', abs(info['power'])) if info['power']!=0 \
#                              else '\phantom{%s}'%ghost)

    info[key] = '$%(cross)s~{}^{%(scUp)s %% }_{%(scDwn)s %% } \,\, {}^{%(pdfUp)s %% }_{%(pdfDwn)s %% }$'%info
    # Set rho if possible
    if not info['cross_8tev'] is None and not info['cross_100tev'] is None:
        info ['rho'] =  '%3d'%int(round(info['cross_100tev']/info['cross_8tev']))
    else:
        info['rho']    = '[--]'
    return info


to_dir = dict([(d.split("_")[0],d) for d in os.listdir(".") if os.path.isdir(d)])

out = "../../Table.tex"
text = ""

parser = re.compile('^@(?P<tag>.*)@(?P<line>.*)')

for line in open("Table_template.tex"):
    # print line
    parsed = parser.match(line)
    if parsed is None:
        text+= line
        continue
 
    tag = parsed.group('tag')
    line = parsed.group('line')

    split_line = line.split("&")

    if tag in to_dir:
        info = get_info(to_dir[tag], key='100tev')
        # print "Info for %s = %s"%(tag,str(info))
    else:
        info = {"cross": 0.0,
                "error":0.0,
                "power": 1,
                "scUp":0,
                "scDwn":0,
                "pdfUp":0,
                "pdfDwn":0,
                "8tev":'N/A',
                "cross_8tev": None,
                "100tev":'N/A',
                "cross_100tev": None,
                "rho":"[--]"}
    if tag+'8TeV' in to_dir:
        info = get_info(to_dir[tag+'8TeV'], info_base=info, key='8tev')
    else:
        info['8tev'] = 'N/A'

    line  = line.replace("\%", "%%")
    new_line = line % info
    text += new_line.replace("%", "\%")

open(out,"w").write(text)







        




