#! /usr/bin/env python

import sys


out     = sys.argv[1]
outname = '%s.mg5'%out 

replace_dict             = {}
replace_dict['name']     = out

# Set your own path here
valentin_MG5path = '.'
valentin_dataDir = './100TeVRareProcesses/DataFile/Tables'

replace_dict['MG5path']  = valentin_MG5path
replace_dict['dataDir']  = valentin_dataDir

processes_decay = [
    {'tag'      : 'hjj',
     'proc_def' : 'h > j j',
     'table_key': 'g.1'
    },
    {'tag'      : 'hjjj',
     'proc_def' : 'h > j j j',
     'table_key': 'g.2'
    },
    {'tag'      : 'haaj',
     'proc_def' : 'h > a a j',
     'table_key': 'g.5'
    },
    {'tag'      : 'zggg',
     'proc_def' : 'z > g g g',
     'table_key': 'g.7'
    },
#    {'tag'      : 'zgggg',
#     'proc_def' : 'z > g g g g',
#     'table_key': 'g.8'
#    },
    {'tag'      : 'hjjjj',
     'proc_def' : 'h > j j j j QED=1',
     'table_key': 'g.3'
    }]

processes_decay_toPhotons = [
    {'tag'      : 'haa',
     'proc_def' : 'h > a a',
     'table_key': 'g.4'
    },
    {'tag'      : 'haaaa',
     'proc_def' : 'h > a a a a',
     'table_key': 'g.6'
    }]

processes_2to1 = [
    {'tag'      : 'pph',
     'proc_def' : 'p p > h',
     'table_key': 'a.1'
    }]

processes_2to2 = [
    {'tag'      : 'pphj',
     'proc_def' : 'p p > h j',
     'table_key': 'a.2'
    },
    {'tag'      : 'ggzg',
     'proc_def' : 'g g > z g',
     'table_key': 'a.4'
    },
    {'tag'      : 'ggag',
     'proc_def' : 'g g > a g',
     'table_key': 'a.6'
    },
    {'tag'      : 'pphh',
     'proc_def' : 'p p > h h',
     'table_key': 'b.1'
    },
    {'tag'      : 'gghz',
     'proc_def' : 'g g > h z',
     'table_key': 'b.4'
    },
    {'tag'      : 'ggzz',
     'proc_def' : 'g g > z z',
     'table_key': 'b.6'
    },
    {'tag'      : 'ggza',
     'proc_def' : 'g g > z a',
     'table_key': 'b.8'
    },
    {'tag'      : 'ggaa',
     'proc_def' : 'g g > a a',
     'table_key': 'b.10'
    },
    {'tag'      : 'ggwpwm',
     'proc_def' : 'g g > w+ w-',
     'table_key': 'b.12'
    },
]

processes_2to3 = [
    {'tag'      : 'pphjj',
     'proc_def' : 'p p > h j j QED=1',
     'table_key': 'a.3'
    },
    {'tag'      : 'ggzgg',
     'proc_def' : 'g g > z g g',
     'table_key': 'a.5'
    },
    {'tag'      : 'ggagg',
     'proc_def' : 'g g > a g g',
     'table_key': 'a.7'
    },
    {'tag'      : 'pphhj',
     'proc_def' : 'p p > h h j',
     'table_key': 'b.2'
    },
    {'tag'      : 'pphaj',
     'proc_def' : 'p p > h a j',
     'table_key': 'b.3'
    },
    {'tag'      : 'gghzg',
     'proc_def' : 'g g > h z g',
     'table_key': 'b.5'
    },
    {'tag'      : 'ggzzg',
     'proc_def' : 'g g > z z g',
     'table_key': 'b.7'
    },
    {'tag'      : 'ggzag',
     'proc_def' : 'g g > z a g',
     'table_key': 'b.9'
    },
    {'tag'      : 'ggaag',
     'proc_def' : 'g g > a a g',
     'table_key': 'b.11'
    },
    {'tag'      : 'ggwpwmg',
     'proc_def' : 'g g > w+ w- g',
     'table_key': 'b.13'
    },
    {'tag'      : 'pphhh',
     'proc_def' : 'p p > h h h',
     'table_key': 'c.1'
    },
    {'tag'      : 'gghhz',
     'proc_def' : 'g g > h h z',
     'table_key': 'c.2'
    },
    {'tag'      : 'gghzz',
     'proc_def' : 'g g > h z z',
     'table_key': 'c.3'
    },
    {'tag'      : 'gghza',
     'proc_def' : 'g g > h z a',
     'table_key': 'c.4'
    },
    {'tag'      : 'pphaa',
     'proc_def' : 'p p > h a a',
     'table_key': 'c.5'
    },
    {'tag'      : 'gghwpwm',
     'proc_def' : 'g g > h w+ w-',
     'table_key': 'c.6'
    },
    {'tag'      : 'ggzzz',
     'proc_def' : 'g g > z z z',
     'table_key': 'c.7'
    },
    {'tag'      : 'ggzza',
     'proc_def' : 'g g > z z a',
     'table_key': 'c.8'
    },
    {'tag'      : 'ggzaa',
     'proc_def' : 'g g > z a a',
     'table_key': 'c.9'
    },
    {'tag'      : 'ggzwpwm',
     'proc_def' : 'g g > z w+ w-',
     'table_key': 'c.10'
    },
    {'tag'      : 'ggawpwm',
     'proc_def' : 'g g > a w+ w-',
     'table_key': 'c.12'
    },
]

processes_epem = [
    {'tag'      : 'epemhh',
     'proc_def' : 'e+ e- > h h',
     'table_key': 'e.2'
    },
    {'tag'      : 'epemggg',
     'proc_def' : 'e+ e- > g g g',
     'table_key': 'e.1'
    },
    {'tag'      : 'epemhhgg',
     'proc_def' : 'e+ e- > h h g g',
     'table_key': 'e.3'
    }
]

processes_Paolo = [
    {'tag'      : 'gghz8TeV',
     'proc_def' : 'g g > h z',
     'table_key': ''
    },
    {'tag'      : 'ggza8TeV',
     'proc_def' : 'g g > z a',
     'table_key': ''
    },
    {'tag'      : 'ggaa8TeV',
     'proc_def' : 'g g > a a',
     'table_key': ''
    },
    {'tag'      : 'gghza8TeV',
     'proc_def' : 'g g > h z a',
     'table_key': ''
    },
    {'tag'      : 'gghzz8TeV',
     'proc_def' : 'g g > h z z',
     'table_key': ''
    },
    {'tag'      : 'gghh8TeV',
     'proc_def' : 'g g > h h',
     'table_key': ''
    },
    {'tag'      : 'gghhz8TeV',
     'proc_def' : 'g g > h h z',
     'table_key': ''
    }
]

processes_Olivier = [
    {'tag'      : 'ggwpwm8TeV',
     'proc_def' : 'g g > w+ w-',
     'table_key': ''
    },
    {'tag'      : 'gghwpwm8TeV',
     'proc_def' : 'g g > h w+ w-',
     'table_key': ''
    },
    {'tag'      : 'ggwpwmz8TeV',
     'proc_def' : 'g g > w+ w- z',
     'table_key': ''
    },
    {'tag'      : 'ggzzz8TeV',
     'proc_def' : 'g g > z z z',
     'table_key': ''
    },
    {'tag'      : 'pphhh8TeV',
     'proc_def' : 'g g > h h h',
     'table_key': ''
    },
    {'tag'      : 'gghaa8TeV',
     'proc_def' : 'g g > h a a',
     'table_key': ''
    },
    {'tag'      : 'gghha8TeV',
     'proc_def' : 'g g > h h a',
     'table_key': ''
    }
]

processes_Valentin = [
    {'tag'      : 'ggh8TeV',
     'proc_def' : 'g g > h',
     'table_key': ''
    },
    {'tag'      : 'ggzz8TeV',
     'proc_def' : 'g g > z z',
     'table_key': ''
    },
    {'tag'      : 'pphjj8TeV',
     'proc_def' : 'p p > h j j QED=1',
     'table_key': ''
    },
    {'tag'      : 'ggazz8TeV',
     'proc_def' : 'g g > a z z',
     'table_key': '',
     'max_npoint':'set max_npoint_for_channel 5'
    },
    {'tag'      : 'gghhhh8TeV',
     'proc_def' : 'g g > h h h h',
     'table_key': ''
    },
    {'tag'      : 'gghaa8TeV',
     'proc_def' : 'g g > h a a',
     'table_key': ''
    },
    {'tag'      : 'ggzaa8TeV',
     'proc_def' : 'g g > z a a',
     'table_key': ''
    }
]


# Choose for which set of process to output a card
#processes = processes_Paolo
#processes = processes_Olivier
processes = processes_Valentin

# For e+ e-
if any(proc['tag'].startswith('epem') or proc['tag'].startswith('haa') for proc in processes):
    header = \
"""import model loop_qcd_qed_sm-paperSetupEW
!echo 'Starting MG5 script %(name)s ...' > ~/MG5_%(name)s_progress.log
set stdout_level 5"""
else:
    header = \
"""import model loop_sm-paperSetup
!echo 'Starting MG5 script %(name)s ...' > ~/MG5_%(name)s_progress.log
set stdout_level 5"""

proc_template_decay = \
"""#######################################
# process %(table_key)s ( %(proc_def)s ) 
#######################################
!echo '... working on %(table_key)s ( %(proc_def)s ) ...' >> ~/MG5_%(name)s_progress.log
generate %(proc_def)s [noborn=QCD]
output %(tag)s -f
launch
set dynamical_scale_choice 5
set MLReductionLib 1
set ImprovePSPoint 1
set lpp1 0
set lpp2 0%(extra)s

set use_syst T
set sys_pdf MSTW2008lo68cl_nf4.LHgrid
!mv %(dataDir)s/%(tag)s %(dataDir)s/OLD_%(tag)s
!mkdir %(dataDir)s/%(tag)s
!cp %(MG5path)s/%(tag)s/crossx.html %(dataDir)s/%(tag)s/crossx.html
!cp %(MG5path)s/%(tag)s/Events/run_01/tag_1_parton_syscalc.log %(dataDir)s/%(tag)s/tag_1_parton_syscalc.log
!echo '... %(table_key)s ( %(proc_def)s ) DONE! ...' >> ~/MG5_%(name)s_progress.log"""

proc_template_decay_toPhotons = \
"""#######################################
# process %(table_key)s ( %(proc_def)s ) 
#######################################
!echo '... working on %(table_key)s ( %(proc_def)s ) ...' >> ~/MG5_%(name)s_progress.log
generate %(proc_def)s [noborn=QED]
output %(tag)s -f
launch
set dynamical_scale_choice 5
set MLReductionLib 1
set ImprovePSPoint 1
set lpp1 0
set lpp2 0%(extra)s

set use_syst T
set sys_pdf MSTW2008lo68cl_nf4.LHgrid
!mv %(dataDir)s/%(tag)s %(dataDir)s/OLD_%(tag)s
!mkdir %(dataDir)s/%(tag)s
!cp %(MG5path)s/%(tag)s/crossx.html %(dataDir)s/%(tag)s/crossx.html
!cp %(MG5path)s/%(tag)s/Events/run_01/tag_1_parton_syscalc.log %(dataDir)s/%(tag)s/tag_1_parton_syscalc.log
!echo '... %(table_key)s ( %(proc_def)s ) DONE! ...' >> ~/MG5_%(name)s_progress.log"""

proc_template = \
"""#######################################
# process %(table_key)s ( %(proc_def)s ) 
#######################################
%(max_npoint)s
!echo '... working on %(table_key)s ( %(proc_def)s ) ...' >> ~/MG5_%(name)s_progress.log
generate %(proc_def)s [QCD]
output %(tag)s -f
launch
set lhc 8
set dynamical_scale_choice 3
set pdlabel lhapdf
set lhaid 23300%(extra)s
set MLReductionLib 1|3
set etaa -1.0
set ptj 20.0
set drjj 0.7
set ptgmin 20.0
set R0gamma 0.4
set xn 1.0
set epsgamma 1.0
set use_syst T
set sys_pdf MSTW2008nlo68cl_nf4.LHgrid
!mv %(dataDir)s/%(tag)s %(dataDir)s/OLD_%(tag)s
!mkdir %(dataDir)s/%(tag)s
!cp %(MG5path)s/%(tag)s/crossx.html %(dataDir)s/%(tag)s/crossx.html
!cp %(MG5path)s/%(tag)s/Events/run_01/tag_1_parton_syscalc.log %(dataDir)s/%(tag)s/tag_1_parton_syscalc.log
!echo '... %(table_key)s ( %(proc_def)s ) DONE! ...' >> ~/MG5_%(name)s_progress.log
"""

proc_template_epem = \
"""#######################################
# process %(table_key)s ( %(proc_def)s ) 
#######################################
!echo '... working on %(table_key)s ( %(proc_def)s ) ...' >> ~/MG5_%(name)s_progress.log
generate %(proc_def)s [noborn=QCD QED]
output %(tag)s -f
launch
set lep 500
set dynamical_scale_choice 2
%(extra)s

set use_syst T
!mv %(dataDir)s/%(tag)s %(dataDir)s/OLD_%(tag)s
!mkdir %(dataDir)s/%(tag)s
!cp %(MG5path)s/%(tag)s/crossx.html %(dataDir)s/%(tag)s/crossx.html
!cp %(MG5path)s/%(tag)s/Events/run_01/tag_1_parton_syscalc.log %(dataDir)s/%(tag)s/tag_1_parton_syscalc.log
!echo '... %(table_key)s ( %(proc_def)s ) DONE! ...' >> ~/MG5_%(name)s_progress.log"""

print "Writing file on %s..."%outname

outfile = open(outname,'w')
res = [header%replace_dict]
for process in processes:
    for key, value in process.items():
        replace_dict[key]=value
    
    if 'max_npoint' not in process:
        replace_dict['max_npoint']='set max_npoint_for_channel 0'

    if any(process['tag'].startswith(t) for t in ['ggzz','ggwpwm']):
        replace_dict['extra'] = '\nset ptheavy 1'        
    elif process['tag'] in ['hjjj','hjjjj']:
        replace_dict['extra'] = '\nset drjj 0.4\nset mmjj 10.0'
    elif process['tag'] in ['haaaa','haaj']:
        replace_dict['extra'] = '\nset draa 0.4\nset mmaa 10.0'
        if process['tag'] == 'haaj':
            replace_dict['extra'] = replace_dict['extra'] + '\nset draj 0.4'
    else:
        replace_dict['extra'] = ''
    

    if replace_dict['tag'].startswith('epem'):
        res.append(proc_template_epem%replace_dict)
    elif replace_dict['tag'].startswith('haa'):
        res.append(proc_template_decay_toPhotons%replace_dict)
    elif replace_dict['tag'].startswith('hjj') or replace_dict['tag'].startswith('zgg'):
        res.append(proc_template_decay%replace_dict)
    else:
        res.append(proc_template%replace_dict)
outfile.write('\n'.join(res))
print "... Done!"
